import 'package:intl/intl.dart';
import 'package:plurdart/plurdart.dart';

import 'user.dart';

// General data structures
// The data returned by getActive and getHistory can be of following nature:
// Friendship request: (requires action from the user)
// {"type": "friendship_request", "from_user": {"nick_name": ...}, "posted": ...}
//
// Friendship pending: (requires action from the user)
// {"type": "friendship_pending", "to_user": {"nick_name": ...}, "posted": ...}
//
// New fan notification:
// {"type": "new_fan", "new_fan": {"nick_name": ...}, "posted": ...}
//
// Friendship accepted notification:
// {"type": "friendship_accepted", "friend_info": {"nick_name": ...}, "posted": ...}
//
// New friend notification:
// {"type": "new_friend", "new_friend": {"nick_name": ...}, "posted": ...}
//
// New private plurk:
// {"type": "private_plurk", "owner": {"nick_name": ...}, "posted": ..., "plurk_id": ...}
//
// User's plurk got liked:
// {"type": "plurk_liked", "from_user": {"nick_name": ...}, "posted": ..., "plurk_id": ..., "num_others": ...}
//
// User's plurk got replurked:
// {"type": "plurk_replurked", "from_user": {"nick_name": ...}, "posted": ..., "plurk_id": ..., "num_others": ...}
//
// User got mentioned in a plurk:
// {"type": "mentioned", "from_user": {"nick_name": ...}, "posted": ..., "plurk_id": ..., "num_others": ..., "response_id": ...}
// response_id may be null if user was mentioned in the plurk and not in a response.
//
// User's own plurk got responded:
// {"type": "my_responded", "from_user": {"nick_name": ...}, "posted": ..., "plurk_id": ..., "num_others": ..., "response_id": ...}
enum AlertType {
  FriendshipRequest,
  FriendshipPending,
  NewFan,
  FriendshipAccepted,
  NewFriend,
  PrivatePlurk,
  PlurkLiked,
  PlurkReplurked,
  Mentioned,
  MyResponded,
  Unknown,
}

class Alert {
  String posted;

  User fromUser;
  User toUser;
  User newFan;
  User friendInfo;
  User newFriend;
  User owner;

  int plurkId;
  String type;

  Alert({this.posted, this.owner, this.plurkId, this.type});

  Alert.fromJson(Map<String, dynamic> json) {
    posted = json['posted'];

    fromUser = json['from_user'] != null ? new User.fromJson(json['from_user']) : null;
    toUser = json['to_user'] != null ? new User.fromJson(json['to_user']) : null;
    newFan = json['new_fan'] != null ? new User.fromJson(json['new_fan']) : null;
    friendInfo = json['friend_info'] != null ? new User.fromJson(json['friend_info']) : null;
    newFriend = json['new_friend'] != null ? new User.fromJson(json['new_friend']) : null;
    owner = json['owner'] != null ? new User.fromJson(json['owner']) : null;

    plurkId = json['plurk_id'];
    type = json['type'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['posted'] = this.posted;

    if (this.owner != null) {
      data['from_user'] = this.fromUser.toJson();
    }

    if (this.owner != null) {
      data['to_user'] = this.toUser.toJson();
    }

    if (this.owner != null) {
      data['new_fan'] = this.newFan.toJson();
    }

    if (this.owner != null) {
      data['friend_info'] = this.friendInfo.toJson();
    }

    if (this.owner != null) {
      data['new_friend'] = this.newFriend.toJson();
    }

    if (this.owner != null) {
      data['owner'] = this.owner.toJson();
    }

    data['plurk_id'] = this.plurkId;
    data['type'] = this.type;
    return data;
  }

  //A convenient enum type method.
  AlertType alertType() {
    if (type != null) {
      switch(type) {
        case 'friendship_request':
          return AlertType.FriendshipRequest;
        case 'friendship_pending':
          return AlertType.FriendshipPending;
        case 'new_fan':
          return AlertType.NewFan;
        case 'friendship_accepted':
          return AlertType.FriendshipAccepted;
        case 'new_friend':
          return AlertType.NewFriend;
        case 'private_plurk':
          return AlertType.PrivatePlurk;
        case 'plurk_liked':
          return AlertType.PlurkLiked;
        case 'plurk_replurked':
          return AlertType.PlurkReplurked;
        case 'mentioned':
          return AlertType.Mentioned;
        case 'my_responded':
          return AlertType.MyResponded;
      }
    }
    return AlertType.Unknown;
  }

  //I don't know why the API has so many user names...
  User getUser() {
    if (fromUser != null) { return fromUser; }
    if (toUser != null) { return toUser; }
    if (newFan != null) { return newFan; }
    if (friendInfo != null) { return friendInfo; }
    if (newFriend != null) { return newFriend; }
    if (owner != null) { return owner; }

    return null;
  }

  //Possible null
  DateTime postedDateTime() {
    if (posted != null) {
      try {
        return DateFormat('EEE, d MMM yyyy HH:mm:ss vvv')
            .parseUTC(posted)
            .toLocal();
      } catch (e) {
        print(e.toString());
        return null;
      }
    }
    return null;
  }

}